#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
from astropy.io import fits
from os import path

import mgefit
from mgefit.mge_fit_1d import mge_fit_1d
from scipy.special import gammaincinv

#check modify
def main():



    rad,me=np.genfromtxt("Blackout.txt",unpack=True)
    #rad,me=np.genfromtxt("Blueout.txt",unpack=True)
    #rad,me=np.genfromtxt("Redout.txt",unpack=True)
    #rad,me=np.genfromtxt("Greenout.txt",unpack=True)
    #rad,me=np.genfromtxt("Magentaout.txt",unpack=True)




    lrad= np.log10(rad)
    lme= np.log10(me)
     


    print("\nFitting 1-dim profile-----------------------------------\n")
    counts,sigma = fit_1d(rad,me)

    print('mge done formatting to output file')


    parfile="mse1dGALFIT.txt"

    # printing files

    mgeoutfile="mgegas.txt"


    fout1 = open(parfile, "w")
    fout2 = open(mgeoutfile, "w")


    outline2 = "# Mag Sig(pixels) FWHM(pixels) Re(pixels) q angle \n"
    fout2.write(outline2)


    totGauss = len(counts)

    #param values

    index = 0
    magzpt = 25
    exptime = 1
    anglegass = 0

    fit = 1
    serfit = 0
    skyfit = 0
    sky = 0

    Z=0

    image = "galaxy.fits"
    rmsname = "galaxy-rms.fits"
    psfname = "psf.fits"
    maskfile="mask.fits"
    outname = "galaxy"


    consfile="constraints.txt"

    T1 = "{}".format(image)
    T2 = outname + "-mge.fits"
    T3 = "{}".format(rmsname)

    xlo = 1
    ylo = 1

    xhi = 2000
    yhi = 2000 

    xpeak = 1000
    ypeak = 1000


    convbox = 100
    scale = 1

    K = gammaincinv(1,0.5)

    PrintHeader(fout1, T1, T2, T3, psfname, 1, maskfile, consfile, xlo, xhi, ylo,
                yhi, convbox, convbox, magzpt, scale, scale, "regular", 0, 0)




    print("total number of gaussians by mge_fit_sectors: ", totGauss)

    while index < totGauss: 


        TotCounts = counts[index]
        SigPix =  sigma[index]
        qobs = 1

        TotCounts = float(TotCounts)
        SigPix = float(SigPix)


        #C0 = TotCounts/(2*np.pi*qobs*SigPix**2)
        C0 = TotCounts/(np.sqrt(2*np.pi)*SigPix)

        Ftot = 2*np.pi*qobs*SigPix**2*C0

        mgemag = magzpt  + 2.5*np.log10(exptime)  - 2.5*np.log10(Ftot)
        #mgemag = magzpt  + 2.5*np.log10(exptime)  - 2.5*np.log10(TotCounts)

        FWHM = 2.35482*SigPix

        h  = np.sqrt(2) * SigPix
             
        Re = (K**(0.5))*h

        outline = "Mag: {:.2f}  Sig: {:.2f}  FWHM: {:.2f} Re: {:.2f}  q: {:.2f} angle: {:.2f} \n".format(mgemag, SigPix, FWHM, Re, qobs, anglegass)
        print(outline)

        outline2 = "{:.2f} {:.2f} {:.2f} {:.2f} {:.2f} {:.2f} \n".format(mgemag, SigPix, FWHM, Re,  qobs, anglegass)

        fout2.write(outline2)

        PrintSersic(fout1, index+1, xpeak + 1, ypeak + 1, mgemag, Re, 0.5, qobs, anglegass, Z, fit, serfit)

        index+=1

    PrintSky(fout1, index+1, sky, Z, skyfit)
    fout1.close()
    fout2.close()

    print("Done. Gaussians are stored in {}, and {} for galfit format ".format(mgeoutfile,parfile))


def fit_1d(x,y):
    """
    Usage example for mge_fit_1d().
    This example reproduces Figure 3 in Cappellari (2002)
    It takes <1s on a 2.5 GHz computer

    """
    #n = 300  # number of sampled points
    #x = np.geomspace(0.01, 300, n)  # logarithmically spaced radii
    #y = (1 + x)**-4  # The profile must be logarithmically sampled!
    plt.clf()
    m = mge_fit_1d(x, y, ngauss=16, plot=True)
    plt.pause(1)  # allow the plot to appear in certain situations

    (counts, sigma) = m.sol
    
    return counts, sigma

def PrintHeader(hdl, A, B, C, D, E, F, G, xlo, xhi, ylo, yhi, convx, convy, J, platedx, platedy, O, P, S):
    "print GALFIT header in a file"

    # k Check
    # print to filehandle
    # the header for GALFIT

    lineZ = "==================================================================================================\n"
    lineX = "# IMAGE PARAMETERS \n"
    lineA = "A) {}                                   # Input Data image (FITS file)                            \n".format(A)
    lineB = "B) {}                                   # Output data image block                                 \n".format(B)
    lineC = "C) {}                                   # Sigma image name (made from data if blank or \"none\")  \n".format(C)
    lineD = "D) {}                                   # Input PSF image and (optional) diffusion kernel         \n".format(D)
    lineE = "E) {}                                   # PSF fine sampling factor relative to data               \n".format(E)
    lineF = "F) {}                                   # Bad pixel mask (FITS image or ASCII coord list)         \n".format(F)
    lineG = "G) {}                                   # File with parameter constraints (ASCII file)            \n".format(G)
    lineH = "H) {} {} {} {}                          # Image region to fit (xmin xmax ymin ymax)               \n".format(xlo, xhi, ylo, yhi)
    lineI = "I) {} {}                                # Size of the convolution box (x y)                       \n".format(convx, convy)
    lineJ = "J) {}                                   # Magnitude photometric zeropoint                         \n".format(J)
    lineK = "K) {} {}                                # Plate scale (dx dy). \[arcsec per pixel\]               \n".format(platedx, platedy)
    lineO = "O) {}                                   # Display type (regular, curses, both)                    \n".format(O)
    lineP = "P) {}                                   # Choose 0=optimize, 1=model, 2=imgblock, 3=subcomps      \n".format(P)
    lineS = "S) {}                                   # Modify/create objects interactively?                    \n".format(S)
    lineY = " \n"

    line0 = "# INITIAL FITTING PARAMETERS                                                     \n"
    line1 = "# \n"
    line2 = "#   For object type, allowed functions are:                                      \n"
    line3 = "#       nuker, sersic, expdisk, devauc, king, psf, gaussian, moffat,             \n"
    line4 = "#       ferrer, powsersic, sky, and isophote.                                    \n"
    line5 = "# \n"
    line6 = "#  Hidden parameters will only appear when they're specified:                    \n"
    line7 = "#      C0 (diskyness/boxyness),                                                  \n"
    line8 = "#      Fn (n=integer, Azimuthal Fourier Modes),                                  \n"
    line9 = "#      R0-R10 (PA rotation, for creating spiral structures).                     \n"
    line10 = "# \n"

    line11 = "# column 1:  Parameter number                                                               \n"
    line12 = "# column 2:                                                                                 \n"
    line13 = "#          -- Parameter 0:    the allowed functions are: sersic, nuker, expdisk             \n"
    line14 = "#                             edgedisk, devauc, king, moffat, gaussian, ferrer, psf, sky    \n"
    line15 = "#          -- Parameter 1-10: value of the initial parameters                               \n"
    line16 = "#          -- Parameter C0:   For diskiness/boxiness                                        \n"
    line17 = "#                             <0 = disky                                                    \n"
    line18 = "#                             >0 = boxy                                                     \n"
    line19 = "#          -- Parameter Z:    Outputting image options, the options are:                    \n"
    line20 = "#                             0 = normal, i.e. subtract final model from the data to create \n"
    line21 = "#                             the residual image                                            \n"
    line22 = "#                             1 = Leave in the model -- do not subtract from the data       \n"
    line23 = "#                                                                                           \n"
    line24 = "# column 3: allow parameter to vary (yes = 1, no = 0)                                       \n"
    line25 = "# column 4: comment                                                                         \n"
    line26 = " \n"

    line27 = "==================================================================================================\n"

    hdl.write(lineZ)
    hdl.write(lineX)
    hdl.write(lineA)
    hdl.write(lineB)
    hdl.write(lineC)
    hdl.write(lineD)
    hdl.write(lineE)
    hdl.write(lineF)
    hdl.write(lineG)
    hdl.write(lineH)
    hdl.write(lineI)
    hdl.write(lineJ)
    hdl.write(lineK)
    hdl.write(lineO)
    hdl.write(lineP)
    hdl.write(lineS)
    hdl.write(lineY)

    hdl.write(line0)
    hdl.write(line1)
    hdl.write(line2)
    hdl.write(line3)
    hdl.write(line4)
    hdl.write(line5)
    hdl.write(line6)
    hdl.write(line7)
    hdl.write(line8)
    hdl.write(line9)
    hdl.write(line10)

    hdl.write(line11)
    hdl.write(line12)
    hdl.write(line13)
    hdl.write(line14)
    hdl.write(line15)
    hdl.write(line16)
    hdl.write(line17)
    hdl.write(line18)
    hdl.write(line19)
    hdl.write(line20)
    hdl.write(line21)
    hdl.write(line22)
    hdl.write(line23)
    hdl.write(line24)
    hdl.write(line25)
    hdl.write(line26)
    hdl.write(line27)

    return True


def PrintSky(hdl, ncomp, sky, Z, fit):
    "Print GALFIT sky function to filehandle"

    # k Check

    line00 = "# Object number: {}                                                             \n".format(ncomp)
    line01 = " 0)      sky            #    Object type                                        \n"
    line02 = " 1) {}         {}       # sky background        [ADU counts]                    \n".format(sky, fit)
    line03 = " 2) 0.000      0        # dsky/dx (sky gradient in x)                           \n"
    line04 = " 3) 0.000      0        # dsky/dy (sky gradient in y)                           \n"
    line05 = " Z) {}                  # Skip this model in output image?  (yes=1, no=0)       \n".format(Z)
    line06 = "\n"
    line07 = "================================================================================\n"

    hdl.write(line00)
    hdl.write(line01)
    hdl.write(line02)
    hdl.write(line03)
    hdl.write(line04)
    hdl.write(line05)
    hdl.write(line06)
    hdl.write(line07)

    return True


def PrintSersic(hdl, ncomp, xpos, ypos, magser, reser, nser, axratser, angleser, Z, fit, serfit):
    "print GALFIT Sersic function to filehandle"
    # k Check

    # print to filehandle
    # a sersic function given
    # by the parameters

    line00 = "# Object number: {}                                                             \n".format(
            ncomp)
    line01 = " 0)     sersic               #  Object type                                     \n"
    line02 = " 1) {:.2f}  {:.2f}  {}  {}            #  position x, y     [pixel]                       \n".format(
        xpos, ypos, fit, fit)
    line03 = " 3) {:.2f}       {}              #  total magnitude                                 \n".format(
        magser, fit)
    line04 = " 4) {:.2f}       {}              #  R_e         [Pixels]                            \n".format(
            reser, fit)
    line05 = " 5) {}       {}              #  Sersic exponent (deVauc=4, expdisk=1)           \n".format(
        nser, serfit)
    #line05 = " 5) {}       {}              #  Sersic exponent (deVauc=4, expdisk=1)           \n".format(
    #    nser, fit)
    line06 = " 6)  0.0000       0           #  ----------------                                \n"
    line07 = " 7)  0.0000       0           #  ----------------                                \n"
    line08 = " 8)  0.0000       0           #  ----------------                                \n"
    line09 = " 9) {:.2f}       {}              #  axis ratio (b/a)                                \n".format(
        axratser, fit)
    line10 = "10) {:.2f}       {}              #  position angle (PA)  [Degrees: Up=0, Left=90]   \n".format(
        angleser, fit)
    lineZ = " Z) {}                       #  Skip this model in output image?  (yes=1, no=0) \n".format(
        Z)
    line11 = "\n"

    hdl.write(line00)
    hdl.write(line01)
    hdl.write(line02)
    hdl.write(line03)
    hdl.write(line04)
    hdl.write(line05)
    hdl.write(line06)
    hdl.write(line07)
    hdl.write(line08)
    hdl.write(line09)
    hdl.write(line10)
    hdl.write(lineZ)
    hdl.write(line11)

    return True



#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
    main()




import argparse



from galfitools.sky.GalfitSky import galfitSky

from galfitools.sky.Sky import sky
from galfitools.sky.SkyDs9 import SkyDs9
from galfitools.sky.SkyRing import SkyRing


from galfitools.shell.prt import printWelcome

def mainGalfitSky():

    printWelcome()
    parser = argparse.ArgumentParser(description="computes the sky using GALFIT")

    parser.add_argument("image", help="the image file")
    parser.add_argument("mask", help="the GALFIT mask file")


    parser.add_argument("-s","--scale", type=float, help="the plate scale. default = 1", default=1)

    parser.add_argument("-zp","--mgzpt", type=float, help="the magnitud zero point. default=25",default = 25)

    parser.add_argument("-x","--xpos", type=float, help="the x position. default=1",default = 1)
    parser.add_argument("-y","--ypos", type=float, help="the y position. default=1",default = 1)

    parser.add_argument("-is","--initsky", type=float, help="the initial sky value default=0",default = 0)


    args = parser.parse_args()

    imgname =  args.image
    maskfile = args.mask 

    mgzpt = args.mgzpt 
    scale = args.scale

    X = args.xpos
    Y = args.ypos

    initsky = args.initsky


    galfitSky(imgname, maskfile, mgzpt, scale, X, Y, initsky)



def mainSky():

    printWelcome()

    parser = argparse.ArgumentParser(description="computes sky from a ds9 region box file")

    parser.add_argument("image", help="the image file")
    parser.add_argument("maskfile", help="the GALFIT Mask image file ")
    parser.add_argument("Ds9regFile", help="the DS9 box region file")
 

   
    args = parser.parse_args()

    imgname = args.image 
    maskimage = args.maskfile 
    filereg = args.Ds9regFile


    mean, sig = sky(imgname, maskimage, filereg)

    print("Sky within 3 sigma:") 

    print("mean sky: {:.3f} ".format(mean))
    print("std sky: {:.3f} ".format(sig))


def mainSkyDs9():


    printWelcome()

    parser = argparse.ArgumentParser(description="SkyDs9: computes sky background from a Ds9 region file: Box, Ellipses and Polygons ")

    parser.add_argument("ImageFile", help="the image file where the photometry will be computed")
    parser.add_argument("RegFile", help="the DS9 region file")


    parser.add_argument("-m","--mask", type=str, help="the mask file")

    parser.add_argument("-ol","--outliers", action="store_true", help="Removes the top 80%% and botttom 20%% of the pixel values to compute sky within ring")
 

    args = parser.parse_args()

    ImageFile = args.ImageFile 
    RegFile = args.RegFile 
    maskfile = args.mask
    outliers = args.outliers


    mean, sig = SkyDs9(ImageFile, RegFile, maskfile, outliers) 



    print("mean sky: {:.3f} ".format(mean))
    print("std sky: {:.3f} ".format(sig))


def mainSkyRing():
    """Computes the sky using rings """


    printWelcome()

    parser = argparse.ArgumentParser(description="SkyRing: computes the sky using gradient over rings")

    # required arguments
    parser.add_argument("Image",help="Fits image of the objects")

    parser.add_argument("Ds9regFile", help="the DS9 ellipse region file")


    parser.add_argument("-m","--mask", type=str, help="Fits Mask image")


    parser.add_argument("-c","--center", action="store_true", help="use the center of the ellipse. Otherwise it will use the (x,y) position with the highest value of the ellipse")


    parser.add_argument("-ol","--outliers", action="store_true", help="Removes the top 80%% and botttom 20%% of the pixel values to compute sky within ring")
  
    # arguments with inputs

   
    parser.add_argument("-w","--width", type=int, help="width of the ring for the gradient method. Default = 20. ",default=20)





    args = parser.parse_args()

    image = args.Image
    mask = args.mask
    ds9regfile = args.Ds9regFile
    width = args.width
    center = args.center
    outliers = args.outliers


    ##end input


    print("Major axis of ellipse is used as initial radius.") 


    mean, std, median, rad = SkyRing(image, mask, ds9regfile, width, center, outliers)




    line="Total sky:  mean = {:.2f}; std={:.2f}; median = {:.2f} at radius {:.2f} ".format(mean,std,median, rad)
    print(line)







import argparse

import numpy as np

from collections import Counter

from galfitools.galin.getStar import getStar
from galfitools.galin.initgal import InitGal
from galfitools.galin.MaskDs9 import maskDs9
from galfitools.galin.getBoxSizeDs9 import getBoxSizeDs9
from galfitools.galin.MakePSF import makePSF
from galfitools.galin.galfit import Galfit
from galfitools.galin.galfit import galfitLastFit 

from galfitools.galin.getSersic import getSersic

from galfitools.galin.galfit import GalComps, GalHead

from galfitools.mge.mge2galfit import PrintHeader, PrintSky, PrintSersic
from galfitools.mge.mge2galfit import mge2gal

from galfitools.galin.MakeMask import makeMask
from galfitools.galin.MaskSky import skyRem
from galfitools.galin.xy2fits import xy2fits

import subprocess as sp

from galfitools.galin.checkGalFile import checkFile 


from galfitools.galin.imarith import imarith 



from galfitools.shell.prt import printWelcome


import sys

def mainGetStar():

    printWelcome()

    parser = argparse.ArgumentParser(description="gets a image slice centered on the object peak")

    parser.add_argument("image", help="the image file to obtain the slice")
    parser.add_argument("Ds9regFile", help="the DS9 ellipse region file containing the star")
    parser.add_argument("size", type=int, help="the size of the new image in pixels")
    parser.add_argument("-c","--center", action="store_true", 
                        help="uses the center given in DS9 region file," + 
                        "otherwise it will find the x,y peaks within DS9 ellipse")
    parser.add_argument("-s","--sky", type=float, 
                        help="the sky background to be removed")
    parser.add_argument("-o","--out", type=str, 
                        help="the image output.",default="star.fits")

    parser.add_argument("-sig","--sigma", type=str, 
                        help="introduce the sigma image")

    parser.add_argument("-so","--sigout", type=str, 
                        help="the sigma image output.",default="sigma.fits")



    args = parser.parse_args()

    image = args.image
    regfile = args.Ds9regFile 
    imsize = args.size
    center = args.center

    sky = args.sky
    imout = args.out

    sigma = args.sigma
    sigout = args.sigout

    getStar(image, regfile, imsize, center, sky, imout, sigma, sigout)

    print("Done. Object fits file: {} created ".format(imout))

    if sigma:
        print("Done. sigma fits file: {} created ".format(sigout))


def mainInitGal(): 


    printWelcome()

   #parser argument section

    parser = argparse.ArgumentParser(description="Creates GALFIT's input files with different initial parameters")

    parser.add_argument("inFile", help="the galfit file galfit.XX ")
    parser.add_argument("-n","--number", type=int, help="the number of files generated. Default = 1",default=1)
        
    parser.add_argument("-p3","--param3", nargs=2, type=float, help="range of values to give to the 3) model's parameter in format [min max] ",)
    parser.add_argument("-p4", "--param4", nargs=2, type=float, help="range of values to give to the 4) model's parameter in format [min max] ")
    parser.add_argument("-p5", "--param5", nargs=2, type=float, help="range of values to give to the 5) model's parameter in format [min max] ")
    parser.add_argument("-p6", "--param6", nargs=2, type=float, help="range of values to give to the 6) model's parameter in format  [min max]")
    parser.add_argument("-p7", "--param7", nargs=2, type=float, help="range of values to give to the 7) model's parameter in format [min max]")
    parser.add_argument("-p8", "--param8", nargs=2, type=float, help="range of values to give to the 8) model's parameter in format [min max]")
    parser.add_argument("-p9", "--param9", nargs=2, type=float, help="range of values to give to the 9) model's parameter in format [min max]")
    parser.add_argument("-p10", "--param10", nargs=2, type=float, help="range of values to give to the 10) model's parameter in format [min max]")



    parser.add_argument("-nc","--numcomp", type=int, help="the component number which parameters will be changed")


    args = parser.parse_args()


    GalfitFile = args.inFile
    number = args.number
    param3 = args.param3
    param4 = args.param4
    param5 = args.param5
    param6 = args.param6
    param7 = args.param7

    param8 = args.param8
    param9 = args.param9
    param10 = args.param10


    numcomp = args.numcomp


    InitGal(GalfitFile, number, param3, param4, param5, param6, param7, param8, param9, param10, numcomp)


    print("rungalfit.sh has been created")


def mainMakeMask():


    printWelcome()

    parser = argparse.ArgumentParser(description="creates mask file from a catalog of Sextractor")

    parser.add_argument("Sexfile", help="Sextractor catalog file ")
    parser.add_argument("ImageFile", help="Image file")

    parser.add_argument("-o","--maskout", type=str, help="the output mask file name  ",default='masksex.fits')
    parser.add_argument("-sf","--satds9", type=str, help="ds9 saturation file",default='ds9sat.reg')
        
    parser.add_argument("-s","--scale", type=float, help="scale factor to increase the ellipses. Default=1",default = 1)
 


    args = parser.parse_args()


    sexfile = args.Sexfile 
    image = args.ImageFile  
    maskfile = args.maskout  
    scale = args.scale 
    satfileout = args.satds9





    makeMask(sexfile, image, maskfile, scale, satfileout)
    

    print("Done. Mask image created ")


def mainMaskDs9(): 

    printWelcome()

    parser = argparse.ArgumentParser(description="creates (or modify) a mask image for GALFIT using Ds9 regions such as Boxes, Ellipses and Polygons ")

    parser.add_argument("MaskFile", help="the Mask image file to modify or create")
    parser.add_argument("RegFile", help="the DS9 region file")

    parser.add_argument("-f","--fill", type=int, help="the value in counts to fill into the Ds9 regions. Default = 0 (remove)",default=0)

    parser.add_argument("-i","--image", type=str, help="image to obtain the size  ")


    parser.add_argument("-b","--border", action="store_true", help="Mask the borders when their value is zero")
    parser.add_argument("-bv","--borValue",default=0,type=float, help="value of the border if it is different from zero")


    parser.add_argument("-sm","--skymean", type=float, help="sky mean to be used as a patch of sky")
    parser.add_argument("-sd","--skystd", type=float, help="sky standard deviation to be used with skymean as a patch of sky",default=1)


    args = parser.parse_args()

    MaskFile = args.MaskFile 
    RegFile = args.RegFile 
    fill = args.fill
    image = args.image

    bor_flag = args.border
    borValue = args.borValue

    skymean = args.skymean
    skystd = args.skystd

    maskDs9(MaskFile, RegFile, fill, image, bor_flag, borValue, skymean=skymean, skystd=skystd) 

    print('Done. Mask {} created (or modified)'.format(MaskFile))



def mainMaskSky(): 


    printWelcome()

    parser = argparse.ArgumentParser(description="creates a mask image for GALFIT using original image and sky mean and sigma")


    parser.add_argument("ImageFile", help="original data image ")
    parser.add_argument("MaskFile", help="Name of the new Mask file")
    parser.add_argument("-sm","--skymean",default=0,type=float, help="mean of the sky background")
    parser.add_argument("-ss","--skysigma",default=0,type=float, help="sigma of the sky background")
    parser.add_argument("-ns","--numbersig",default=1,type=float, help="number of times that the sigma of the sky will be multiplied to remove the sky background")

    parser.add_argument("-r","--region", type=str, help="Ds9 region to remove from mask")

    parser.add_argument("-b","--border", action="store_true", help="Mask the borders when their value is zero")

    parser.add_argument("-bv","--borValue",default=0,type=float, help="value of the border if it is different from zero")
    args = parser.parse_args()

    image = args.ImageFile 
    mask = args.MaskFile
    sky_mean = args.skymean
    sky_sig = args.skysigma
    nsig = args.numbersig

    region = args.region

    bor_flag = args.border
    borValue = args.borValue


    skyRem(image, mask, sky_mean, sky_sig, nsig, borValue, bor_flag)


    if region:
        maskDs9(mask, region, 0, None, False, 0)  #remove ds9 region if provided


    print('Done. Mask file {} created'.format(mask))


def mainxy2fits(): 

    printWelcome()

    parser = argparse.ArgumentParser(description="code to convert ASCII x,y positions to FTIS mask")

    parser.add_argument("ImageFile", help="The Image file ")
    parser.add_argument("AsciiMask", help="The ascii file with the x,y positions ")
    parser.add_argument("-c","--val", type=int, help="the value in counts for the masked pixels. Default = 1",default=1)
 
    args = parser.parse_args()




    ImageFile= args.ImageFile
    AsciiFile= args.AsciiMask
    Value = args.val 

    xy2fits().MakeFits(ImageFile, AsciiFile, Value)

    print("Ascii -> Fits done ")



#console scripts
def maincheckFile() -> None: 

    printWelcome()
    #reading arguments parsing

    parser = argparse.ArgumentParser(description = "checkFile: check that the parameters and file names inside the GALFIT input file are correct")


    # required arguments
    parser.add_argument("GalfitFile", help = "GALFIT input File ")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance in pixels among components. Default = 10", default=10)


    ## parsing variables

    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis

   
    headinfo, galax, mag, freepar = checkFile(galfitFile, dis)

    testflag = False

    if not(headinfo.inputimageflag):
        line="File {} not found ".format(headinfo.inputimage)
        print(line)
        testflag = True

    #if not(headinfo.outimageflag):
    #    line="File {} not found ".format(headinfo.outimage)
    #    print(line)
    #    testflag = True

    if not(headinfo.sigimageflag):
        line="File {} not found ".format(headinfo.sigimage)
        print(line)
        #testflag = True
        line="GALFIT will create a sigma file "
        print(line)
 
    if not(headinfo.psfimageflag):
        line="File {} not found ".format(headinfo.psfimage)
        print(line)
        testflag = True

    if (headinfo.psfimageflag):
        if not(headinfo.convxflag):
            line="Warning: x-size of convolution is smaller than psf image x-axis"
            print(line)
            testflag = True

        if not(headinfo.convyflag):
            line="Warning: y-size of convolution is smaller than psf image y-axis"
            print(line)
            testflag = True




    if not(headinfo.maskimageflag):
        line="File {} not found ".format(headinfo.maskimage)
        print(line)
        testflag = True

    if not(headinfo.constraintsflag):
        line="File {} not found ".format(headinfo.constraints)
        print(line)
        testflag = True

    if not(headinfo.xsizeflag):
        line="format of size in x-axis not valid. xmax < xmin"
        print(line)
        testflag = True

    if not(headinfo.ysizeflag):
        line="Format of size in y-axis not valid. ymax < ymin"
        print(line)
        testflag = True



    if not(testflag):
        print("No issues with the header information")
    else:
        print("There are some issues with the header information. Check messages above.")

    print('Total number of model components: ', len(galax))
    print('Total number of galaxies: ', len(np.unique(galax)))


    print('Components per galaxy: ')

    cnt = Counter(galax)


    Flux = 10**((25 - mag)/2.5) #25 is just a constant to avoid small numbers. I substract it later

    for idx,item in enumerate(np.unique(galax)): 
        totcomp = cnt[item]  
        maskgal  = galax == item 
         

        totFlux = Flux[maskgal].sum()

        totmag = -2.5*np.log10(totFlux) + 25 

        line="galaxy {} has {} components and a total mag of: {:.2f} ".format(int(item), totcomp, totmag)
        print(line)
 

    line="Total number of free parameters: {} ".format(freepar)
    print(line)
 




def mainGetBoxSizeDs9(): 


    printWelcome()

   #parser argument section

    parser = argparse.ArgumentParser(description="Computes the Box size from a Ds9 region file for galfit header")

    parser.add_argument("RegFile", help="Ds9 region file containing the box region ")
    args = parser.parse_args()


    RegFile = args.RegFile

    xmin, xmax, ymin, ymax = getBoxSizeDs9(RegFile)


    line="xmin, xmax, ymin, ymax: {} {} {} {} ".format(xmin, xmax, ymin, ymax)
    print(line)
 

def maingetSersic(): 

    printWelcome()

    parser = argparse.ArgumentParser(description="prints the Sersic function from DS9 ellipse region")


    parser.add_argument("Image", help = "image fits file")
    parser.add_argument("RegFile", help = "DS9 ellipse region file")


    parser.add_argument("-zp","--zeropoint", type=float, help="The value of the zero point. Default = 25 ",default=25)


    parser.add_argument("-sk","--sky", default=0, type=float, help="Sky background value to be removed from image before photometry. Default = 0")


    parser.add_argument("-bt","--bulgetot", type=float, help="Bulge to total ratio. If set it will print two sersics: one for the bulge and the other for the disk")

    parser.add_argument("-c","--center", action="store_true", help="takes center of ds9 region file")

    parser.add_argument("-n","--noprint", action="store_true", help="avoids to print Sersic functionts to stdout")
    parser.add_argument("-m","--mask", type=str, help="the mask file")

    parser.add_argument("-b","--bards9", type=str, help="DS9 ellipse region file that containts the bar region. bulgetot flag must be activated")

    args = parser.parse_args()

    image = args.Image
    regfile = args.RegFile
    center = args.center
    maskfile = args.mask
    zeropoint = args.zeropoint
    sky = args.sky
    bulgetot = args.bulgetot
    noprint = args.noprint
    bards9 = args.bards9


    galcomps = getSersic(image, regfile, center, maskfile, zeropoint, sky, noprint, bulgetot, bards9)



def main_imarith() -> None: 

    printWelcome()
    #reading arguments parsing

    parser = argparse.ArgumentParser(description = "imarith: makes arithmetics operations on images")

    # required arguments
    parser.add_argument("ImageFile", help = "The input image")


    parser.add_argument("-o", "--output", type=str, default="output.fits", help = "The output image")

    parser.add_argument("-i", "--image2", type=str, help = "second input image to make arithmetic operations with ImageFile. Image2 must be of the same size of ImageFile. If this second image is provided it will make operations indicated by arithmetic flag ignoring its constant input")

    parser.add_argument("-a","--add", type=float, help="add constant to image pixels" )
    parser.add_argument("-d","--div", type=float, help="divide all pixels by constant" )
    parser.add_argument("-m","--mul", type=float, help="multiply all pixels by constant" )
    parser.add_argument("-s","--sub", type=float, help="substract constant to all pixels")



    ## parsing variables

    args = parser.parse_args()

    ImageFile = args.ImageFile
    output = args.output
    image2 = args.image2

    add = args.add
    mul = args.mul
    div = args.div
    sub = args.sub

    imarith(ImageFile, output, image2, add, mul, div, sub)


    print("done")

def mainMakePSF():

    printWelcome()

    parser = argparse.ArgumentParser(description="Makes a PSF model of a star using MGE")

    parser.add_argument("image", help="the image file where it contains the star to be modelled ")
    parser.add_argument("GalfitFile", help="GALFIT file to obtain the header options")

    parser.add_argument("Ds9regFile", help="the DS9 ellipse region file containing the star to model")

    parser.add_argument("-c","--center", action="store_true", 
                        help="uses the center given in DS9 region file, " + 
                        "otherwise it will find the (x,y) peak within DS9 ellipse")

    parser.add_argument("-o","--out", type=str, 
                        help="the PSF model image",default="psf.fits")

    parser.add_argument("-sig","--sigma", type=str, 
                        help="introduce the sigma image")

    parser.add_argument("-t","--twist", action="store_true", help="uses twist option for mge ")

    parser.add_argument("-ng","--numgauss", type=int, help="number of gaussians that will be used for galfit.")

    #######################
    args = parser.parse_args()
    ##

    image = args.image
    regfile = args.Ds9regFile 
    center = args.center

    psfout = args.out

    sigma = args.sigma


    galfitFile = args.GalfitFile
    twist = args.twist

    numgauss = args.numgauss 

    #######################


    makePSF(galfitFile, image, regfile, center, psfout, sigma, twist, numgauss) 






#!/usr/bin/env python3

import numpy as np
import sys
import os.path
from astropy.io import fits


from matplotlib.path import Path


from galfitools.galin.MaskDs9 import GetAxis 


def photDs9(ImageFile, RegFile, maskfile, zeropoint, sky): 



    if not os.path.exists(ImageFile):

        print ('image filename does not exist!' )
        sys.exit()

    if not os.path.exists(RegFile):
        print ('%s: reg filename does not exist!' %(sys.argv[2]))
        sys.exit()



    (ncol, nrow) = GetAxis(ImageFile)

    exptime = GetExpTime(ImageFile)

    hdu = fits.open(ImageFile)

    Image = hdu[0].data

    hdu.close()


 


    #removing sky background from image

    Image = Image - sky



    v0 = []
    v1 = []
    v2 = []
    v3 = []
    v4 = []
    v5 = []

    tupVerts = []
    Pol = []


    f1 = open(RegFile,'r')

    lines = f1.readlines()

    f1.close()

    flag = False
    flagpoly = False
    #reading reg file
    for line in lines:


        b1 = line.split("(")
        p = line.split(",")

        x1 = p[0]
        if b1[0] == "ellipse":

            x0 = "ellipse"
            x2 = x1[8:]
            flag = True

        if b1[0] == "box":

            x0 = "box"
            x2 = x1[4:]
            flag = True


        if b1[0] == "polygon":

            polv = line.split(")")
            pol, ver = polv[0].split("(")


            points = ver.split(",")

            N = len(points)

            i=0

            x = np.array([])
            y = np.array([])

            x = x.astype(float)
            y = y.astype(float)

            verts = []

            #make tuple for vertices
            while i < N:

                verts = verts + [(round(float(points[i])-1),round(float(points[i+1])-1)) ]
                i = i + 2

            flagpoly = True

        if (flag == True):

            x3 = p[4]
            x4 = x3[:-2]

            v0.append(x0)
            v1.append(float(x2) - 1)
            v2.append(float(p[1]) - 1)
            v3.append(float(p[2]))
            v4.append(float(p[3]))
            v5.append(float(x4))

            flag = False

        if (flagpoly  == True):

            Pol.append(pol) 
            tupVerts.append(verts)
            flagpoly = False

    obj  = np.array(v0)
    xpos = np.array(v1) 
    ypos = np.array(v2)
    rx = np.array(v3)
    ry = np.array(v4)
    angle = np.array(v5)

    Pol = np.array(Pol)

    totFlux = 0

    if (maskfile == "none") or (maskfile == "None"): 
        maskfile = None

    # mask file
    if maskfile:
        errmsg="file {} does not exist".format(maskfile)
        assert os.path.isfile(maskfile), errmsg

        hdu2 = fits.open(maskfile)
        maskimage = hdu2[0].data
        maskb = np.array(maskimage,dtype=bool)
        invmask = np.logical_not(maskb)
        invmask = invmask*1
        Image = Image*invmask
        hdu2.close()


    


    for idx, item in enumerate(obj):

        #get Flux

        if obj[idx] == "ellipse":

            ellFlux = FluxEllip(Image, xpos[idx], ypos[idx], rx[idx], 
                                ry[idx], angle[idx], ncol, nrow)

            totFlux = totFlux + ellFlux

        if obj[idx] == "box":


            boxFlux = FluxBox(Image, xpos[idx], ypos[idx], rx[idx], 
                            ry[idx], angle[idx], ncol, nrow)


            totFlux = totFlux + boxFlux

    for idx, item in enumerate(Pol):

        #converting ellipse from DS9 ell to kron ellipse

        if Pol[idx] == "polygon":

            polFlux = FluxPolygon(Image, tupVerts[idx], ncol, nrow)



            totFlux = totFlux + polFlux

    mag = -2.5*np.log10(totFlux/exptime) + zeropoint



    return mag, exptime


def FluxEllip(Image, xpos, ypos, rx, ry, angle, ncol, nrow):
    "obtain flux from  an ellipse region in an image"

    xx, yy, Rkron, theta, e = Ds9ell2Kronell(xpos,ypos,rx,ry,angle)
    (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, e, ncol, nrow)
    flux = FluxKron(Image, xx, yy, Rkron, theta, e, xmin, xmax, ymin, ymax)

    return flux 

def FluxPolygon(Image, tupVerts, ncol, nrow):
    "obtainn flux from a polygon region in an image"


    x, y = np.meshgrid(np.arange(ncol), np.arange(nrow)) # make a canvas with coordinates
    x, y = x.flatten(), y.flatten()
    points = np.vstack((x,y)).T 
    p = Path(tupVerts) # make a polygon




    grid = p.contains_points(points)
    mask = grid.reshape(nrow, ncol) # now you have a mask with points inside a polygon


    flux = Image[mask].sum()



    return flux 




def FluxBox(Image, xpos,ypos,rx,ry,angle,ncol,nrow):
    "obtain the flux from a box region in an image"


    anglerad = angle*np.pi/180
    beta = np.pi/2 - anglerad

    lx = (rx/2)*np.cos(anglerad) - (ry/2)*np.cos(beta)
    lx2 = (rx/2)*np.cos(anglerad) + (ry/2)*np.cos(beta)
    ly = (rx/2)*np.sin(anglerad) + (ry/2)*np.sin(beta)
    ly2 = (rx/2)*np.sin(anglerad) - (ry/2)*np.sin(beta)


    v1x = round(xpos - lx)  
    v1y = round(ypos - ly) 


    v2x = round(xpos - lx2) 
    v2y = round(ypos - ly2) 


    v3x = round(xpos + lx)  
    v3y = round(ypos + ly) 


    v4x = round(xpos + lx2)
    v4y = round(ypos + ly2)



    Verts = [(v1x,v1y),(v2x,v2y),(v3x,v3y),(v4x,v4y)]



    x, y = np.meshgrid(np.arange(ncol), np.arange(nrow)) # make a canvas with coordinates
    x, y = x.flatten(), y.flatten()
    points = np.vstack((x,y)).T 

    p = Path(Verts) # make a polygon

    grid = p.contains_points(points)

    mask = grid.reshape(nrow, ncol) # now you have a mask with points inside a polygon


    flux = Image[mask].sum()

    return flux 





def MakeBoxBack(Image,fill,xpos,ypos,rx,ry,angle,ncol,nrow):
    "Make a box in an image"

    xlo = xpos - rx/2 - 1 
    xhi = xpos + rx/2 + 1 

    ylo = ypos - ry/2 - 1
    yhi = ypos + ry/2 

    if xlo  < 1:
        xlo = 1
    if ylo  < 1:
        ylo = 1

    if xhi  > ncol:
        xhi = ncol - 1 
 
    if yhi  > nrow:
        yhi = nrow - 1
 
    xlo=int(np.round(xlo))
    xhi=int(np.round(xhi))

    ylo=int(np.round(ylo))
    yhi=int(np.round(yhi))


    Verts = [(xlo,yhi),(xhi,yhi),(xhi,ylo),(xlo, ylo)]



    x, y = np.meshgrid(np.arange(ncol), np.arange(nrow)) # make a canvas with coordinates
    x, y = x.flatten(), y.flatten()
    points = np.vstack((x,y)).T 

    p = Path(Verts) # make a polygon

    grid = p.contains_points(points)

    mask = grid.reshape(nrow, ncol) # now you have a mask with points inside a polygon


    Image[mask] = fill




    #Image[ylo - 1: yhi + 1, xlo - 1: xhi + 1] = fill




    return Image







def Ds9ell2Kronell(xpos,ypos,rx,ry,angle):


    if rx >= ry:

        q = ry/rx
        e = 1 - q
        Rkron = rx
        theta = angle
        xx = xpos
        yy = ypos
    else:
        q = rx/ry
        e = 1 - q
        Rkron = ry
        theta = angle + 90
        xx = xpos
        yy = ypos
 
     
    return xx, yy, Rkron, theta, e 



def FluxKron(imagemat, x, y, R, theta, ell, xmin, xmax, ymin, ymax):
    "This subroutine obtain flux from a Kron ellipse within a box defined by: xmin, xmax, ymin, ymax"


    xmin = int(xmin)
    xmax = int(xmax)
    ymin = int(ymin)
    ymax = int(ymax)

    q = (1 - ell)
    bim = q * R

    theta = theta * np.pi / 180  # Rads!!!

    ypos, xpos = np.mgrid[ymin - 1 : ymax + 1, xmin - 1: xmax +1]

    dx = xpos - x
    dy = ypos - y

    landa = np.arctan2(dy, dx)

    mask = landa < 0
    if mask.any():
        landa[mask] = landa[mask] + 2*np.pi

    landa = landa - theta

    angle = np.arctan2(np.sin(landa) / bim, np.cos(landa) / R)

    xell = x + R * np.cos(angle) * np.cos(theta) - bim * \
        np.sin(angle) * np.sin(theta)
    yell = y + R * np.cos(angle) * np.sin(theta) + bim * \
        np.sin(angle) * np.cos(theta)

    dell = np.sqrt((xell - x)**2 + (yell - y)**2)
    dist = np.sqrt(dx**2 + dy**2)

    mask = dist <= dell
    flux = imagemat[ypos[mask], xpos[mask]].sum()

    return flux 


def GetSize(x, y, R, theta, ell, ncol, nrow):
    "this subroutine get the maximun"
    "and minimim pixels for Kron and sky ellipse"
    # k Check
    q = (1 - ell)
    bim = q * R

    theta = theta * (np.pi / 180)  # rads!!

    # getting size

    constx =  np.sqrt((R**2)*(np.cos(theta))**2 + (bim**2)*(np.sin(theta))**2)
    consty =  np.sqrt((R**2)*(np.sin(theta))**2 + (bim**2)*(np.cos(theta))**2)



    xmin = x - constx                       
    xmax = x + constx
    ymin = y - consty
    ymax = y + consty
                    

    mask = xmin < 1
    if mask.any():
        xmin = 1

    mask = xmax > ncol
    if mask.any():
        xmax = ncol - 1 

    mask = ymin < 1
    if mask.any():
        ymin = 1

    mask = ymax > nrow
    if mask.any():
        ymax = nrow - 1

    return (round(xmin), round(xmax), round(ymin), round(ymax))


def GetExpTime(Image):
    # k Check
    "Get exposition time from the image"


    try:
        hdu = fits.open(Image)
        exptime = hdu[0].header["EXPTIME"]
        hdu.close()
    except: 
        exptime = 1
    return float(exptime)


  
#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  main()





#!/usr/bin/env python3


import numpy as np
from astropy.io import fits
import sys
import subprocess as sp
import os.path

import os
import re

from galfitools.galin.MaskDs9 import GetAxis 
from galfitools.galin.MaskDs9 import checkCompHDU

from galfitools.galin.galfit import Galfit
from galfitools.galin.galfit import galfitLastFit 
from galfitools.galin.galfit import galPrintHeader, galPrintSky, galPrintComp

from galfitools.galin.getStar import getStar

from galfitools.mge.mge2galfit import mge2gal, GetInfoEllip, Ds9ell2Kronellv2


def makePSF(galfitFile: str, image: str, regfile: str, center: bool, psfout: str, sigma: str, twist: bool, numgauss: int)-> None: 


    #######################
    # reading options from galfit header

    galfit = Galfit(galfitFile)
    head = galfit.ReadHead()
    galsky = galfit.ReadSky()

    inputimage = head.inputimage

    imageout = head.outimage
    magzpt = head.mgzpt
    maskfile = head.maskimage

    sky = galsky.sky

    scale = head.scale
    psfile = head.psfimage

    sigfile = head.sigimage

    convbox =  head.convx
    convboxy = head.convy

    xlo = head.xmin
    ylo = head.ymin
    xhi = head.xmax
    yhi = head.ymax
 
    imsize = head.xmax - head.xmin +  1

    #################

    #inputimage of galfit header will be the output of getStar
    getStar(image, regfile, imsize, center, sky,  inputimage, sigma, sigfile)



    #checking the center position
    even = False

    if imsize % 2 == 0:
        even = True

    
    if even:
        lx = imsize/2

        xpeak = int(lx + 1) 
        ypeak = int(lx + 1) 
        
    else:
        lx = imsize/2

        xpeak = int(lx + 0.5) 
        ypeak = int(lx + 0.5) 


    ####

    psf = 0
    gauss = None
    freeser = False 
    freesky = False 

    xypos = [xpeak, ypeak]  

    #calling to these two functions to obtain eps and theta
    obj_trash, xpos_trash, ypos_trash, rx_trash, ry_trash, angle_trash = GetInfoEllip(regfile)
    xx_trash, yy_trash, Rkron_trash, theta, eps = Ds9ell2Kronellv2(xpos_trash,ypos_trash,rx_trash,ry_trash,angle_trash)


    ellip = eps 
    posang = theta 
    regmgefile = None



    outname = mge2gal(galfitFile, regmgefile, center, psf, twist, gauss, 
            freeser, freesky, numgauss, xypos=xypos, ellip = ellip, posang = posang) 



    print("calling GALFIT to fit MGE")

    rungal = "galfit  {}".format("mseGALFIT.txt")
    errgal = sp.run([rungal], shell=True, stdout=sp.PIPE, stderr=sp.PIPE,
                universal_newlines=True)



    #check if galfit failed
    if re.search("Doh!", errgal.stdout): 
        print("ERROR: GALFIT has been unable to find a solution")
        print("Try to reduce the number of gaussians with -ng option")
        print("Exiting now")
        sys.exit(1)


    lastfit_file = galfitLastFit(".")

    galfit = Galfit(lastfit_file)
    head = galfit.ReadHead()
    galcomps = galfit.ReadComps()
    galsky = galfit.ReadSky()


    #printing output file to create psf model
    fout1 = open("psfmodel.txt", "w")

    head.outimage = psfout
    head.P = 1 

    galPrintHeader(fout1, head)



    index = 0


    for index, item in enumerate(galcomps.N):


            galPrintComp(fout1, index+1, index, galcomps)




    galsky.skip = 1
 
    galPrintSky(fout1, index+1, galsky)


    fout1.close()


    print("calling GALFIT again to create final model")

    rungal = "galfit  {}".format("psfmodel.txt")
    errgal = sp.run([rungal], shell=True, stdout=sp.PIPE, stderr=sp.PIPE,
                universal_newlines=True)

    print("Done. PSF model file: ", psfout)


    print("Check the image, model and residual: ",outname)



#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################




#!/usr/bin/env python3


import numpy as np
from astropy.io import fits
import sys
import subprocess as sp
import os.path

import argparse

from galfitools.galin.MaskDs9 import GetAxis 

from galfitools.galin.MaskDs9 import checkCompHDU




def getPeak(image: str, regfile: str, center: bool, maskfile: str)-> None: 
    '''return the coordinates of the peak given an DS9 region file'''

    #root_ext = os.path.splitext(image)
    #timg= root_ext[0]

    if maskfile:

        errmsg="file {} does not exist".format(maskfile)
        assert os.path.isfile(maskfile), errmsg
        hdu = fits.open(maskfile)
        mask = hdu[0].data
        maskb=np.array(mask,dtype=bool)
        maskbt=maskb.T
        hdu.close()

    else:
        mask=np.array([])


    #mask = np.array([]) #empty for the moment

    i = 0

    hdu = fits.open(image)
    img = hdu[i].data

    hdu.close()
    
    (ncol, nrow) = GetAxis(image)
    
    obj, xpos, ypos, rx, ry, angle = GetInfoEllip(regfile)
    xx, yy, Rkron, theta, eps = Ds9ell2Kronellv2(xpos,ypos,rx,ry,angle)


    if center:
        print('center of DS9 ellipse region will be used')
        xpeak, ypeak = round(xpos), round(ypos)
    else:        
        (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, eps, ncol, nrow)

        xpeak, ypeak = GetPmax(img, mask, xmin, xmax, ymin, ymax)


    #print("object found at ", xpeak + 1, ypeak + 1)

    axratio = 1 - eps

    return xpeak + 1, ypeak + 1, axratio, theta





####################################################
####################################################
#####################################################



def GetInfoEllip(regfile):

    if not os.path.exists(regfile):
        print ('%s: reg filename does not exist!' %(regfile))
        sys.exit()

    f1 = open(regfile,'r')

    lines = f1.readlines()

    f1.close()

    flag = False
    found = False

    #reading reg file
    for line in lines:


        b1 = line.split("(")
        p = line.split(",")

        x1 = p[0]
        if b1[0] == "ellipse":

            x0 = "ellipse"
            x2 = x1[8:]
            flag = True
            found = True

        if (flag == True):
            x3 = p[4]
            x4 = x3[:-2]


            v0 = x0

            v1 = float(x2)
            v2 = float(p[1])
            v3 = float(p[2])
            v4 = float(p[3])
            v5 = float(x4)

            flag = False

    if found:
        obj  = v0
        xpos = v1
        ypos = v2
        rx = v3
        ry = v4
        angle = v5

        if rx >= ry:
            axratio = ry/rx 
            eps = 1 - axratio 
            theta = angle + 90 
        else:
            axratio = rx/ry 
            eps = 1 - axratio 
            theta = angle 

        return obj, xpos, ypos, rx, ry, angle
        #return eps, theta, xpos, ypos
    else:
        print("ellipse region was not found in file. Exiting.. ")
        sys.exit()

    return 0, 0, 0, 0



def Ds9ell2Kronellv2(xpos,ypos,rx,ry,angle):

    if rx >= ry:

        q = ry/rx
        e = 1 - q
        Rkron = rx
        theta = angle - 90
        xx = xpos
        yy = ypos
    else:
        q = rx/ry
        e = 1 - q
        Rkron = ry
        theta = angle# + 90
        xx = xpos
        yy = ypos
 
     
    return xx, yy, Rkron, theta, e 


def GetSize(x, y, R, theta, ell, ncol, nrow):
    "this subroutine get the maximun"
    "and minimim pixels for Kron and sky ellipse"
    # k Check
    q = (1 - ell)
    bim = q * R

    theta = theta * (np.pi / 180)  # rads!!

    # getting size

    xmin = x - np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    xmax = x + np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    ymin = y - np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    ymax = y + np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    mask = xmin < 1
    if mask.any():
        xmin = 1

    mask = xmax > ncol
    if mask.any():
        xmax = ncol

    mask = ymin < 1
    if mask.any():
        ymin = 1

    mask = ymax > nrow 
    if mask.any():
        ymax = nrow

    
    return (xmin, xmax, ymin, ymax)



def GetPmax(image, mask, xmin, xmax, ymin, ymax):


    xmin = int(xmin)
    xmax = int(xmax)
    ymin = int(ymin)
    ymax = int(ymax)

    chuckimg = image[ymin - 1:ymax, xmin - 1:xmax]
    if mask.any():
        chuckmsk = mask[ymin - 1:ymax, xmin - 1:xmax]

        invmask = np.logical_not(chuckmsk)

        invmask = invmask*1

        chuckimg = chuckimg*invmask

    maxy, maxx = np.where(chuckimg == np.max(chuckimg))
    
    xpos = maxx[0] + xmin - 1
    ypos = maxy[0] + ymin - 1

    return (xpos, ypos)



#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################

if __name__ == '__main__':
    mainGetStar()



#!/usr/bin/env python3

import numpy as np
import sys
import os
import stat
import subprocess as sp
import os.path
from astropy.io import fits
import scipy
import scipy.special
import matplotlib.pyplot as plt


import argparse

#introducir sky box y sky ring
#use maskds9 para obtener los pixeles de las regiones

def sky(imgname, maskimage, filereg) -> None:



    xlo,xhi,ylo,yhi = GetRegionDs9(filereg)
    ################################################
    ################################################


    hdu = fits.open(imgname)
    imgdat = hdu[0].data.astype(float)
    hdu.close()

    hdu = fits.open(maskimage)
    maskdat = hdu[0].data
    hdu.close()

    imgdat  = imgdat[ylo - 1:yhi, xlo - 1:xhi]
    maskdat = maskdat[ylo - 1:yhi, xlo - 1:xhi]


    maskdat=np.array(maskdat,dtype=bool)



    mask = maskdat == False

    img=imgdat[mask]  



    #print("mean sky: {:.3f} ".format(img.mean()))
    #print("std sky: {:.3f} ".format(img.std()))
    #print("rms sky: {:.3f} ".format(rms(img)))


    #print("Excluding the top and bottom 20%:") 
 
    flatimg = img.flatten()  
    flatimg.sort()

    tot = len(flatimg)

    top = round(.8*tot)
    bot = round(.2*tot)

    img2 = flatimg[bot:top]

    mean = img2.mean()
    sig = img2.std()


    return mean, sig


    #print("mean sky: {:.3f} ".format(mean))
    #print("std sky: {:.3f} ".format(sig))


    #img3=img2.copy()

    #mask2  = np.abs(img3 - mean) <= 3* sig 

    #msky = img3[mask2].mean()
    #ssky = img3[mask2].std()

    #return msky, ssky 
    



def rms(array):
   return np.sqrt(np.mean(array ** 2))


def GetRegionDs9(filein):
    "Get the size (xmin,xmax,ymin,ymax) from ds9 region file "

    xhi=0
    xlo=0
    ylo=0
    yhi=0

    with open(filein) as f_in:

        lines = (line.rstrip() for line in f_in) # All lines including the blank ones
        lines = (line.split('#', 1)[0] for line in lines) # remove comments
        lines = (line.rstrip() for line in lines)   # remove lines containing only comments
        lines = (line for line in lines if line) # Non-blank lines

        for line in lines:

            (chunks)=line.split(' ')
            if (chunks[0] != "image" and chunks[0] != "physical" and chunks[0] != "global"  ):

                (box,info)=line.split('(')

                if(box == "box"):

                    (xpos,ypos,xlong,ylong,trash)=info.split(',')

                    xpos=float(xpos)
                    ypos=float(ypos)
                    xlong=float(xlong)
                    ylong=float(ylong)


                    xlo = xpos - xlong/2
                    xhi = xpos + xlong/2

                    ylo = ypos - ylong/2
                    yhi = ypos + ylong/2



    xlo=int(round(xlo))
    xhi=int(round(xhi))
    ylo=int(round(ylo))
    yhi=int(round(yhi))


    return xlo,xhi,ylo,yhi




#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
    main()





import argparse


from galfitools.shell.prt import printWelcome

from galfitools.galout.getRads import getBreak
from galfitools.galout.getRads import getBreak2
from galfitools.galout.getRads import getFWHM
from galfitools.galout.getRads import getKappa
from galfitools.galout.getRads import getReComp
from galfitools.galout.getRads import getSlope
from galfitools.galout.getRads import getBulgeRad

from galfitools.galout.getMissingLight import getMissLight

from galfitools.galout.getN import getN

from galfitools.galout.showcube import displayCube

from galfitools.galout.PhotDs9 import photDs9 

from galfitools.galout.getCOW import getCOW

from galfitools.galout.getPeak import getPeak

from galfitools.galout.fitlog2csv import log2csv 


from galfitools.galin.galfit import Galfit

from galfitools.mge.mge2galfit import GetInfoEllip



def mainPhotDs9(): 

    printWelcome()

    parser = argparse.ArgumentParser(description="computes photometry from a Ds9 region file: Box, Ellipses and Polygons ")

    parser.add_argument("ImageFile", help="the image file where the photometry will be computed")
    parser.add_argument("RegFile", help="the DS9 region file")

    parser.add_argument("-zp","--zeropoint", type=float, help="The value of the zero point. Default = 25 ",default=25)


    parser.add_argument("-m","--mask", type=str, help="the mask file")

    parser.add_argument("-sk","--sky", default=0, type=float, help="Sky background value to be removed from image before photometry. Default = 0")




    args = parser.parse_args()

    ImageFile = args.ImageFile 
    RegFile = args.RegFile 
    maskfile = args.mask
    zeropoint = args.zeropoint
    sky = args.sky


    mag, exptime = photDs9(ImageFile, RegFile, maskfile ,zeropoint, sky)


    line = "the exposition time is is: {} \n".format(exptime)
    print(line)

    line = "the magnitude from the ds9 region is: {:.2f} \n".format(mag)
    print(line)



#console scripts
def mainGetBreak() -> None: 

    printWelcome()
    #reading argument parsing

    parser = argparse.ArgumentParser(description = "getBreak: gets the break radius from a set of Sersics ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-a","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components")


    parser.add_argument("-ni","--numinitial", type=int, help="Number of component where it'll obtain the initial parameter to search break radius or to generated random initial radius. ", default=2)


    parser.add_argument("-q","--quick", action="store_true", help='evaluate in position only (given by -ni parameter') 



    parser.add_argument("-r","--random", type=int, help="Number of random radius as initial parameters to search for the minimum. It will generated random radius from 0 to effective radius of the component indicated by parameter -ni ")


    parser.add_argument("-p","--plot", action="store_true", help='makes plot of double derivative vs. radius ') 
    parser.add_argument("-rx","--ranx",nargs=2, type=float, help="provide a range for the plot x-axis: xmin - xmax ")




    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis

    inicomp = args.numinitial

    quick = args.quick
    random = args.random

    num_comp =  args.numcomp

    angle = args.angle


    ranx = args.ranx
    plot = args.plot


    rbreak, N, theta = getBreak(galfitFile, dis, inicomp, quick, random, num_comp, angle, plot, ranx)
        
    print('number of model components: ',N)

    line = 'Using a theta value of: {:.2f} degrees\n'.format(theta)
    print(line)


    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    rbreak_arc = rbreak*plate
 

    line = 'The break radius is {:.2f} pixels or {:.2f} "  \n'.format(rbreak,rbreak_arc)
    print(line)



def mainGetBreak2():
    '''gets the break radius from a set of Sersics'''


    printWelcome()

    parser = argparse.ArgumentParser(description = "getBreak2: gets the break radius from a set of Sersics ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)
    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-a","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components")


    parser.add_argument("-p","--plot", action="store_true", help='makes plot of double derivative vs. radius ') 

    parser.add_argument("-rx","--ranx",nargs=2, type=float, help="x-axis range to search for the Break radius: xmin - xmax ")


    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis
    angle = args.angle
    num_comp =  args.numcomp
    plot = args.plot
    ranx = args.ranx

    rbreak, N, theta =  getBreak2(galfitFile, dis, angle, num_comp, plot, ranx)


    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    rbreak_arc = rbreak*plate
 



    print('number of model components: ',N)

    line = 'Using a theta value of: {:.2f} degrees\n'.format(theta)
    print(line)


    line = 'The break radius is {:.2f} pixels or {:.2f} " \n'.format(rbreak, rbreak_arc)
    print(line)




def mainFWHM() -> None: 


    printWelcome()

    parser = argparse.ArgumentParser(description = "getFWHM: gets the FWHM from a set of Sersics ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)
    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-a","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components")





    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis
    angle = args.angle


   
    num_comp =  args.numcomp

    fwhm, N, theta = getFWHM(galfitFile, dis, angle, num_comp)


    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    fwhm_arc = fwhm*plate
 


    print('number of model components: ',N)

    line = 'Using a theta value of : {:.2f} degrees\n'.format(theta)
    print(line)

    line = 'The FWHM is {:.2f} pixels or {:.2f} " \n'.format(fwhm, fwhm_arc)
    print(line)




#console scripts
def mainKappa() -> None: 

    printWelcome()
    #reading argument parsing

    parser = argparse.ArgumentParser(description = "getKappa: gets the Kappa radius from a set of Sersics ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-a","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components")


    parser.add_argument("-ni","--numinitial", type=int, help="Number of component where it'll obtain the initial parameter to search break radius or to generated random initial radius. ", default=2)


    parser.add_argument("-q","--quick", action="store_true", help='evaluate in position only (given by -ni parameter') 


    parser.add_argument("-r","--random", type=int, help="Number of random radius as initial parameters to search for the minimum. It will generated random radius from 0 to effective radius of the component indicated by parameter -ni ")


    parser.add_argument("-p","--plot", action="store_true", help='makes plot of maximum curvature vs. radius ') 
    parser.add_argument("-rx","--ranx",nargs=2, type=float, help="provide a range for x-axis: xmin - xmax ")




    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis

    inicomp = args.numinitial

    quick = args.quick
    random = args.random
    angle = args.angle

    ranx = args.ranx
    plot = args.plot


    num_comp =  args.numcomp


    rkappa, N, theta = getKappa(galfitFile, dis, inicomp, quick, random, angle, num_comp, plot, ranx) 

    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    rkappa_arc = rkappa*plate
 

    print('number of model components: ',N)

    line = 'The Kappa radius  is {:.2f} pixels or {:.2f} " \n'.format(rkappa, rkappa_arc)
    print(line)


def mainGetReComp() -> None: 

    printWelcome()
    #reading arguments parsing
    parser = argparse.ArgumentParser(description = "getReComp: gets the effective radius from a set of Sersics ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)
    parser.add_argument("-fr","--fracrad", type=float, 
                        help="fraction of light radius. default=.5 for effective radius ", default=.5)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-pa","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components. Angle measured from Y-Axis as same as GALFIT. ")



    args = parser.parse_args()


    galfitFile = args.GalfitFile
    dis = args.dis
    eff = args.fracrad
    num_comp =  args.numcomp
    angle = args.angle



    EffRad,  totmag, meanme, me, N, theta = getReComp(galfitFile, dis, eff, angle, num_comp)


    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    EffRad_arc = EffRad*plate
 


    print('number of model components: ', N)

    line = 'Using a theta value of : {:.2f} degrees \n'.format(theta)
    print(line)

    line = 'Total Magnitude of the galaxy: {:.2f} \n'.format(totmag)
    print(line)

    line = 'Surface brightness at radius of {:.0f}% of light (\u03BCr): {:.2f} mag/\" \n'.format(eff*100,me)
    print(line)


    line = 'Mean Surface Brightness at effective radius (<\u03BC>e): {:.2f} mag/\" \n'.format(meanme)
    if(eff == 0.5):
        print(line)

    line = 'The radius at {:.0f}% of light is {:.2f} pixels or {:.2f} " \n'.format(eff*100, EffRad, EffRad_arc)
    print(line)




def maingetSlope() -> None: 

    printWelcome()
    #reading argument parsing

    parser = argparse.ArgumentParser(description = "getSlope: gets the slope radius from a set of Sersics ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-a","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components")


    parser.add_argument("-s","--slope", type=float, 
                        help="value of slope to find. default=.5 ", default=.5)



    parser.add_argument("-p","--plot", action="store_true", help='makes plot of first derivative vs. radius ') 
    parser.add_argument("-rx","--ranx",nargs=2, type=float, help="provide a range for x-axis: xmin - xmax ")




    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis

    slope = args.slope

    num_comp =  args.numcomp

    angle = args.angle

    ranx = args.ranx
    plot = args.plot



    rgam, N, theta = getSlope(galfitFile, dis, slope, angle, num_comp, plot, ranx)



    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    rgam_arc = rgam*plate
 



    print('number of model components: ',N)

    line = 'Using a theta value of : {:.2f} degrees\n'.format(theta)
    print(line)

    line = 'The radius with slope {:.2f} is {:.2f} pixels, or {:.2f} " \n'.format(slope,rgam,rgam_arc)
    print(line)



#check modify
#console scripts
def maingetN() -> None: 

    printWelcome()
    #reading arguments parsing

    parser = argparse.ArgumentParser(description = "getN: computes the Sersic index from surface brightness at effective radius")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-pa","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components. Angle measured from Y-Axis as same as GALFIT. ")


    parser.add_argument("-rf","--radfrac", type=float, help="fraction of light radius. Default = .2 ", default=.2)



    parser.add_argument("-p","--plot", action="store_true", help='makes plot of Sersic index vs. fraction of light ') 


    parser.add_argument("-c","--const", type=float, 
                        help="Substract constant from plot. Default = 0", default=0)



    ## parsing variables

    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis


   
    num_comp =  args.numcomp

    frac = args.radfrac

    angle = args.angle

    plot = args.plot
    const = args.const


    sersic, meanser, stdser, totmag, N, theta = getN(galfitFile, dis, frac, angle, num_comp, plot, const)


    print('number of model components: ', N)

    line = 'Using a theta value of : {:.2f} degrees \n'.format(theta)
    print(line)

    line = 'Total Magnitude of the galaxy: {:.2f} \n'.format(totmag)
    print(line)

    line = 'Sersic index with the method of Mean Surface Brightness at effective radius: {:.2f}  \n'.format(sersic)
    print(line)


    line = 'Sersic index with the method of fraction of light (at different radius)  \n'
    print(line)



    line = 'Sersic index mean: {:.2f}  Standard deviation: {:.2f}  '.format(meanser, stdser)
    print(line)




def mainGetBulgeRad() -> None: 
    '''gets the bulge radius or the radius where two models of surface brightness models are equal'''

    printWelcome()
    #reading arguments parsing

    parser = argparse.ArgumentParser(description = "getBulgeRad: gets the bulge radius or the radius where two models of surface brightness models are equal")


    # required arguments
    parser.add_argument("GalfitFile1", help = "Galfit File containing the coreless surface brightness model")
    parser.add_argument("GalfitFile2", help = "Galfit File containing the core surface brightness model")

    #parser.add_argument("gammarad", type=float, help="gamma radius")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-pa","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components. Angle measured from Y-Axis as same as GALFIT. ")


    parser.add_argument("-p","--plot", action="store_true", help='makes plot of GalfitFile1 - GalfitFile2 vs. radius ') 
    parser.add_argument("-rx","--ranx", nargs=2, type=float, help="provide a range for x-axis: xmin - xmax ")


    ## parsing variables

    args = parser.parse_args()

    galfitFile1 = args.GalfitFile1
    galfitFile2 = args.GalfitFile2

    dis = args.dis

    num_comp =  args.numcomp

    angle = args.angle

    ranx = args.ranx
    plot = args.plot


    rbulge, N1, N2, theta = getBulgeRad(galfitFile1, galfitFile2, dis, num_comp, angle, plot, ranx)

    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()

    plate = head.scale 
    rbulge_arc = rbulge*plate
 

    print('number of model components for the bulge: ',N1)
    print('number of model components for the rest of the galaxy: ',N2)


    line = 'Using a theta value of: {:.2f} degrees\n'.format(theta)
    print(line)


    line = 'The bulge radius is {:.2f} pixels or {:.2f} "  \n'.format(rbulge)
    print(line)




def mainMissingLight() -> None: 
    '''gets the missing light from a two models'''

    printWelcome()
    #reading arguments parsing

    parser = argparse.ArgumentParser(description = "getMissLight: computes the missing light from two surface brightness models")


    # required arguments
    parser.add_argument("GalfitFile1", help = "Galfit File containing the coreless surface brightness model")
    parser.add_argument("GalfitFile2", help = "Galfit File containing the core surface brightness model")

    parser.add_argument("rad", type=float, help="upper limit of radius to integrate the missing light in pixels")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)


    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)


    ## parsing variables

    args = parser.parse_args()

    galfitFile1 = args.GalfitFile1
    galfitFile2 = args.GalfitFile2

    dis = args.dis

    num_comp =  args.numcomp

    rad = args.rad



    magmiss, N1, N2 = getMissLight(galfitFile1, galfitFile2, dis, num_comp, rad)


    print('number of model components coreless model: ',N1)
    print('number of model components core model: ',N2)


    line = 'the missing light is {:.2f} mag \n'.format(magmiss)
    print(line)






def mainShowCube():


    printWelcome()
    parser = argparse.ArgumentParser(description="show the cube fits of the galfit output")

    parser.add_argument("cubeimage", help="the cube GALFIT image")
    parser.add_argument("-o","--outimage", type=str, help="the output png file",default="cube.png")


    #### 
    parser.add_argument("-br","--brightness", type=float, 
                        help="brightness of the image. Only for galaxy and model. Default = 0. Preferible range goes from -1 to 1", default=0)
    parser.add_argument("-co","--contrast", type=float, 
                        help="contrast of the image. Only for galaxy and model. Default = 1. Preferible range goes from 0 to 1",default=1)

    parser.add_argument("-cm","--cmap", type=str, help="cmap to be used for the cube image ",default="viridis")
    parser.add_argument("-dpi","--dotsinch", type=int, help="dots per inch used for images files ",default=100)
    parser.add_argument("-s","--scale", type=float, help="plate scale of the image. Default = 1",default=1)
    parser.add_argument("-np","--noplot", action="store_true", help="it doesn\'t show plotting window")

    args = parser.parse_args()

    cubeimage = args.cubeimage
    namecube = args.outimage 
    dpival = args.dotsinch 
    brightness = args.brightness
    contrast = args.contrast 
    cmap = args.cmap 
    scale = args.scale  
    noplot = args.noplot


    displayCube(cubeimage, namecube, dpival, brightness, contrast, cmap, scale, noplot)





def maingetCOW() -> None: 

    printWelcome()
    #reading arguments parsing

    parser = argparse.ArgumentParser(description = "getCOW: plots the curve-of-growth from the galfit.XX file. Only for Sersic functions")


    # required arguments
    parser.add_argument("GalfitFile", help = "GALFIT File containing the Sersics")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)

    parser.add_argument("-pf","--plotfile", type=str, help="name of the plot file", default='cow.png')

    parser.add_argument("-g","--galfitF2", type=str, help="Second GALFIT file to add to the plot (optional) ")


    parser.add_argument("-md","--maxdiff", action="store_true", help="plot the maximum difference between model 1 and 2 (a vertical line) ")


    parser.add_argument("-fr","--fracrad", type=float, 
                        help="fraction of light radius. This is the upper limit of X-Axis. default=.95 ", default=.95)

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-pa","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components. Angle measured from Y-Axis as same as GALFIT. ")



    parser.add_argument("-dpi","--dotsinch", type=int, help="dots per inch used for images files ",default=100)

    ## parsing variables

    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis
    plotfile = args.plotfile


    dpival = args.dotsinch 

    frac = args.fracrad

    maxdiff = args.maxdiff

   
    num_comp =  args.numcomp


    angle = args.angle

    galfitF2 = args.galfitF2

    totmag, N, theta = getCOW(galfitFile, dis, angle, frac, num_comp, plotfile, dpival, galfitF2, maxdiff)


    print('number of model components: ', N)

    line = 'Using a theta value of : {:.2f} degrees \n'.format(theta)
    print(line)

    line = 'Total Magnitude of the galaxy: {:.2f} \n'.format(totmag)
    print(line)


    print("plot file: ", plotfile)


def mainFitlog2CSV(): 

    printWelcome()

    parser = argparse.ArgumentParser(description="converts fit.log file into a comma separated values file")


    parser.add_argument("-o","--fileout", type=str, help="output file",default="fitlog.csv")


    parser.add_argument("-n","--num", type=int, help="number of the fit in fit.log to convert to csv. Default: last fit")



    args = parser.parse_args()

    fileout = args.fileout
    num = args.num


    log2csv(num, fileout)

    print("conversion to CSV file done.")





def maingetPeak(): 

    printWelcome()

    parser = argparse.ArgumentParser(description="Obtains the center, axis ratio and angular position from DS9 region")


    parser.add_argument("Image", help = "image fits file")
    parser.add_argument("RegFile", help = "DS9 ellipse region file")


    parser.add_argument("-c","--center", action="store_true", help="takes center of ds9 region file")
    parser.add_argument("-m","--mask", type=str, help="the mask file")

    args = parser.parse_args()

    image = args.Image
    regfile = args.RegFile
    center = args.center
    maskfile = args.mask

    #zeropoint = args.zeropoint
    #sky = args.sky


    X, Y, AxRat, PA = getPeak(image, regfile, center, maskfile)


    print("peak is at (x, y) = ", X, Y)

    print("axis ratio q = {:.2f} ".format(AxRat))
    print("angular position = {:.2f}".format(PA)) 







#! /usr/bin/env python

import numpy as np
import argparse
import os
from astropy.io import fits
import subprocess as sp
import scipy
import sys
import copy

from scipy.special import gamma, gammainc, gammaincinv

from scipy.optimize import bisect

import matplotlib.pyplot as plt



#console scripts
def main() -> None: 
    '''gets the effective radius from a set of Sersics'''

    #reading argument parsing

    parser = argparse.ArgumentParser(description = "getSlopeDat: gets the slope and store it in a file ")


    # required arguments
    parser.add_argument("GalfitFile", help = "Galfit File containing the Sersics or gaussians components")

    parser.add_argument("-d","--dis", type=int, help="Maximum distance among components", default=10)
    parser.add_argument("-er","--effrad", type=float, 
                        help="percentage of light to compute for radius. default=.5 for effective radius ", default=.5)

    #parser.add_argument("-ser","--sersic", action="store_true", help="uses sersic function for galfit file")

    parser.add_argument("-n","--numcomp", type=int, help="Number of component where it'll obtain center of all components, default = 1 ", default=1)

    parser.add_argument("-a","--angle", type=float, 
                        help="Angle of the major axis of the galaxy. Default= it will take the angle of the last components")


    parser.add_argument("-s","--slope", type=float, 
                        help="value of slope to find. default=.5 ", default=.5)




    args = parser.parse_args()

    galfitFile = args.GalfitFile
    dis = args.dis

    eff = args.effrad
    slope = args.slope

    assert (eff > 0) and (eff <= 1), 'effrad must be a value between 0 and 1'
   
    num_comp =  args.numcomp



    head = ReadHead(galfitFile)


    galcomps = ReadComps(galfitFile)

    #convert all exp, gaussian and de vaucouleurs to Sersic format
    comps = conver2Sersic(galcomps) 



    comps.Flux = 10**((-comps.Mag)/2.5)

    k = gammaincinv(2*comps.Exp, 0.5)

    denom1 = (2*np.pi*comps.Rad**2)*(np.exp(k))
    denom2 = (comps.Exp)*(k**(-2*comps.Exp))
    denom3 = (gamma(2*comps.Exp))*(comps.AxRat) 

    denom = denom1*denom2*denom3 
    
    comps.Ie = comps.Flux/denom



    comps = SelectGal(comps, dis, num_comp)

    #taking the last component position angle for the whole galaxy

    maskgal = (comps.Active == True) 
    if args.angle:
        theta = args.angle
    else:
        theta = comps.PosAng[maskgal][-1]  




    N = numComps(comps,'all')
    print('number of model components: ',N)

    if N == 0:
        print('not enough number of components to compute Re')
        print('exiting..')
        sys.exit(1)

    line = 'Using a theta value of : {:.2f} degrees\n'.format(theta)
    print(line)



    #########################
    ### computing the slope
    #########################


    R = np.arange(0.1,100,.1)

    gam = GetSlope().GalSlope(R, comps, theta) 


    plt.plot(R, gam)
    plt.xlabel("rad")
    plt.xlabel("slope")
    plt.savefig("slope.png")


    #saving data

    stuff = [['nameservers','panel'], ['nameservers','panel']]
    with open("out.txt", "w") as o:
        for idx,item in enumerate(R):
            print("{} {}".format(R[idx], gam[idx]), file=o)



    rgam = GetSlope().FindSlope(comps, theta, slope) 

    line = 'The radius with slope {:.2f} is {:.2f} pixels \n'.format(slope,rgam)
    print(line)


    return None


class GalHead():
    '''store the header of galfit file'''

    '''store the header of galfit file'''

    inputimage = "galaxy.fits"
    outimage = "galaxy-out.fits"
    sigimage = "sig.fits"
    psfimage = "psf.fits"
    psfsamp = 1
    maskimage = "galaxy-mask.fits"
    constraints = "constraints"
    xmin = 0
    xmax = 1
    ymin = 0
    ymax = 1
    convx = 0
    convy = 0
    mgzpt = 25
    scale = 1
    scaley = 1
    display = "regular"
    P = 0
    S = 0
   


def ReadHead(File: str) -> GalHead:
    '''reads header of galfit file'''
    inputf = File 

    galhead = GalHead() # class for header

    maskimage = ""
    #    skylevel=0

    GalfitFile = open(inputf,"r")

    # All lines including the blank ones
    lines = (line.rstrip() for line in GalfitFile)
    lines = (line.split('#', 1)[0] for line in lines)  # remove comments
    # remove lines containing only comments
    lines = (line.rstrip() for line in lines)
    lines = (line for line in lines if line)  # Non-blank lines

    lines = list(lines)
    index = 0

    while index < len(lines):

        #================================================================================
        # IMAGE and GALFIT CONTROL PARAMETERS
        #A) tempfits/A2399-3-2.fits      # Input data image (FITS file)
        #B) A2399-215615.96-m073822.7-337-out.fits      # Output data image block
        #C) tempfits/none-3-2.fits      # Sigma image name (made from data if blank or "none")
        #D) psfs/PSF-1309-721.fits          # Input PSF image and (optional) diffusion kernel
        #E) 1                   # PSF fine sampling factor relative to data
        #F) mask-337            # Bad pixel mask (FITS image or ASCII coord list)
        #G) constraints         # File with parameter constraints (ASCII file)
        #H) 129  809  265  809  # Image region to fit (xmin xmax ymin ymax)
        #I) 60     60           # Size of the convolution box (x y)
        #J) 21.672              # Magnitude photometric zeropoint
        #K) 0.680  0.680        # Plate scale (dx dy)   [arcsec per pixel]
        #O) regular             # Display type (regular, curses, both)
        #P) 0                   # Choose: 0=optimize, 1=model, 2=imgblock, 3=subcomps

        line = lines[index]
        (tmp) = line.split()
        if tmp[0] == "A)":     # input image
            try:
                galhead.inputimage = tmp[1]
            except IndexError:
                galhead.inputimage = "None"


        if tmp[0] == "B)":     # out image
            try:
                galhead.outimage = tmp[1]
            except IndexError:
                galhead.outimage = "None"


        if tmp[0] == "C)":   # sigma 
            try:
                galhead.sigimage = tmp[1]
            except IndexError:
                galhead.sigimage = "None"


        if tmp[0] == "D)":  # psf file 
            try:
                galhead.psfimage = tmp[1]
            except IndexError:
                galhead.psfimage = "None"


        if tmp[0] == "E)":  #psf sampling 
            galhead.psfsamp = int(tmp[1])

        if tmp[0] == "F)":     # mask image
            try:
                galhead.maskimage = tmp[1]
            except IndexError:
                galhead.maskimage = "None"

        if tmp[0] == "G)":  # constraints file 
            try:
                galhead.constraints = tmp[1]
            except IndexError:
                galhead.constraints = "None"



        if tmp[0] == "H)":     # region fit box
            galhead.xmin = int(tmp[1])
            galhead.xmax = int(tmp[2])
            galhead.ymin = int(tmp[3])
            galhead.ymax = int(tmp[4])

        if tmp[0] == "I)":     # convolution size 
            galhead.convx = int(tmp[1])
            galhead.convy = int(tmp[2])

        if tmp[0] == "J)":     # mgzpt
            galhead.mgzpt = float(tmp[1])

        if tmp[0] == "K)":     # plate scale
            galhead.scale = float(tmp[1])
            galhead.scaley = float(tmp[2])

        if tmp[0] == "O)":     # display 
            galhead.display = tmp[1]

        if tmp[0] == "P)":     # optional output 
            galhead.P = int(tmp[1])

        if tmp[0] == "S)":     # optional output 
            galhead.S = int(tmp[1])



        index += 1

    return galhead

### class for Galfit components
class GalComps:
    '''stores the components of galfit file'''
    N = np.array([])
    NameComp = np.array([])        #0)
    PosX = np.array([])            #1)   
    PosY = np.array([])            #2)   
    Mag = np.array([])             #3)
    Rad = np.array([])             #4)
    Exp = np.array([])             #5)
    Exp2 = np.array([])            #6)  for moffat
    Exp3 = np.array([])            #7)  for moffat
                                   #8)  There is No 8 in any galfit model
    AxRat = np.array([])           #9)  AxisRatio
    PosAng = np.array([])          #10) position angle
    skip = np.array([])            #z)  skip model

    Active = np.array([])            #activate component  for galaxy

    Ie = np.array([])            # surface brightness at effective radius
    Flux = np.array([])         # Flux galax  

    # store the flags related to parameters
    FreePosX = np.array([])            #1)   
    FreePosY = np.array([])            #2)   
    FreeMag = np.array([])             #3)
    FreeRad = np.array([])             #4)
    FreeExp = np.array([])             #5)
    FreeExp2 = np.array([])            #6)  for moffat
    FreeExp3 = np.array([])            #7)  for moffat
                                   #8)  There is No 8 in any galfit model
    FreeAxRat = np.array([])           #9)  AxisRatio
    FreePosAng = np.array([])          #10) position angle

def numComps(galcomps: GalComps, name: str) -> int:
    '''obtains the number of components'''


    if name == 'all':
        nummask = (galcomps.Active == True) & ( (galcomps.NameComp == 'sersic') 
                    | (galcomps.NameComp == 'expdisk') | (galcomps.NameComp == 'gaussian')
                    | (galcomps.NameComp == 'devauc'))

    else:
        nummask = (galcomps.Active == True) & (galcomps.NameComp == name)




    N = galcomps.Active[nummask].size

    return N



def ReadComps(File: str) -> GalComps:
    '''reads all the components in the galfit file'''
    galcomps = GalComps()
     
    GalfitFile = open(File,"r")

    # All lines including the blank ones
    lines = (line.rstrip() for line in GalfitFile)
    lines = (line.split('#', 1)[0] for line in lines)  # remove comments
    # remove lines containing only comments
    lines = (line.rstrip() for line in lines)
    lines = (line for line in lines if line)  # Non-blank lines

    lines = list(lines)
    index = 0

    N = 0


    while index < len(lines):

        line = lines[index]
        (tmp) = line.split()

        #init values
        NameComp = "none"
        PosX = 0
        PosY = 0
        Mag = 99
        Rad = 0
        Exp = 0
        Exp2 = 0
        Exp3 = 0
        AxRat = 1
        PosAng = 0
        skip = 0

        flagcomp = False 

        freeX = 1
        freeY = 1
        Magfree = 1
        Radfree = 1
        Expfree = 1
        Exp2free = 1
        Exp3free = 1
        AxRatfree = 1
        PosAngfree = 1



        if (tmp[0] == "0)") and (tmp[1] != 'sky'):

            namec = tmp[1] 
            N = N + 1 
            NameComp = namec
            while (tmp[0] != "Z)"):

                index += 1
                line = lines[index]
                (tmp) = line.split()

                if tmp[0] == "1)":   # center
                    PosX = float(tmp[1])
                    PosY = float(tmp[2])
                    freeX = int(tmp[3])
                    freeY = int(tmp[4])
                if tmp[0] == "3)" :    # mag 
                    Mag = float(tmp[1])
                    Magfree = int(tmp[2])
                if tmp[0] == "4)" :
                    Rad = float(tmp[1])
                    Radfree = int(tmp[2])
                if tmp[0] == "5)" : 
                    Exp = float(tmp[1])
                    Expfree = int(tmp[2])
                if tmp[0] == "6)": 
                    Exp2 = float(tmp[1])
                    Exp2free = int(tmp[2])
                if tmp[0] == "7)":  
                    Exp3 = float(tmp[1])
                    Exp3free = int(tmp[2])
                if tmp[0] == "9)":
                    AxRat = float(tmp[1])
                    AxRatfree = int(tmp[2])
                if tmp[0] == "10)":
                    PosAng = float(tmp[1])
                    PosAngfree = int(tmp[2])
                if tmp[0] == "Z)":
                    skip=int(tmp[1])

            galcomps.PosX = np.append(galcomps.PosX, PosX)
            galcomps.PosY = np.append(galcomps.PosY, PosY)
            galcomps.NameComp = np.append(galcomps.NameComp, NameComp)
            galcomps.N = np.append(galcomps.N, N)
                
            galcomps.Mag = np.append(galcomps.Mag, Mag)
            galcomps.Rad = np.append(galcomps.Rad, Rad)
            galcomps.Exp = np.append(galcomps.Exp, Exp)
            galcomps.Exp2 = np.append(galcomps.Exp2, Exp2)
            galcomps.Exp3 = np.append(galcomps.Exp3, Exp3)
            galcomps.AxRat = np.append(galcomps.AxRat, AxRat)
            galcomps.PosAng = np.append(galcomps.PosAng, PosAng)
            galcomps.skip = np.append(galcomps.skip, skip)
            galcomps.Active = np.append(galcomps.Active, flagcomp)


            galcomps.FreePosX = np.append(galcomps.FreePosX, freeX)
            galcomps.FreePosY = np.append(galcomps.FreePosY, freeY)
            galcomps.FreeMag = np.append(galcomps.FreeMag, Magfree)
            galcomps.FreeRad = np.append(galcomps.FreeRad, Radfree)
            galcomps.FreeExp = np.append(galcomps.FreeExp, Expfree)
            galcomps.FreeExp2 = np.append(galcomps.FreeExp2, Exp2free)
            galcomps.FreeExp3 = np.append(galcomps.FreeExp3, Exp3free)
            galcomps.FreeAxRat = np.append(galcomps.FreeAxRat, AxRatfree)
            galcomps.FreePosAng = np.append(galcomps.FreePosAng, PosAngfree)



        index += 1

    GalfitFile.close()

    return galcomps 


class GalSky:
    '''stores the value of the GALFIT file'''

    sky = 0 
    dskyx = 0
    dskyy = 0
    skip = 0 

    skyfree = 1 
    dskyxfree = 0 
    dskyyfree = 0 
 



def ReadSky(File: str) -> GalSky:
    '''reads the sky value of the galfit file'''
    
    galsky = GalSky()

    GalfitFile = open(File,"r")

    # All lines including the blank ones
    lines = (line.rstrip() for line in GalfitFile)
    lines = (line.split('#', 1)[0] for line in lines)  # remove comments
    # remove lines containing only comments
    lines = (line.rstrip() for line in lines)
    lines = (line for line in lines if line)  # Non-blank lines

    lines = list(lines)
    index = 0

    N = 0


    while index < len(lines):

        line = lines[index]
        (tmp) = line.split()

        #init values
        NameComp = "sky"
        sky = 0
        dskyx = 0
        dskyy = 0
        skip = 0

        freesky = 1
        freedskyx = 0
        freedskyy = 0


        if (tmp[0] == "0)") and (tmp[1] == NameComp):

            namec = tmp[1] 

            while (tmp[0] != "Z)"):

                index += 1
                line = lines[index]
                (tmp) = line.split()

                if tmp[0] == "1)":   # center
                    sky = float(tmp[1])
                    freesky = float(tmp[2])
                if tmp[0] == "2)" :    # mag 
                    dskyx = float(tmp[1])
                    freedskyx = int(tmp[2])
                if tmp[0] == "3)" :
                    dskyy = float(tmp[1])
                    freedskyy = int(tmp[2])
                if tmp[0] == "Z)":
                    skip=int(tmp[1])

            galsky.NameComp = np.append(galcomps.NameComp, NameComp)
                
            galsky.sky = np.append(galsky.sky, sky)
            galsky.dskyx = np.append(galsky.dskyx, dskyx)
            galsky.dskyy = np.append(galsky.dskyy, dskyy)
            galskyskip = np.append(galsky.skip, skip)

            galsky.skyfree = np.append(galsky.skyfree, skyfree)
            galsky.dskyxfree = np.append(galsky.dskyxfree, dskyxfree)
            galsky.dskyyfree = np.append(galsky.dskyyfree, dskyyfree)
 

        index += 1

    GalfitFile.close()

 
    return galsky



def SelectGal(galcomps: GalComps, distmax: float, n_comp: int) -> GalComps:
    '''changes Flag to true for those components who belongs
        to the same galaxy of n_comp'''

    galcomps.Active.fill(False)

    idx = np.where(galcomps.N ==  n_comp)

    assert idx[0].size !=0, 'component not found'

    n_idx = idx[0].item(0)


    galcomps.Active[n_idx] = True #main component

    posx = galcomps.PosX[n_idx] 
    posy = galcomps.PosY[n_idx]      

    dx = galcomps.PosX - posx
    dy = galcomps.PosY - posy

    dist = np.sqrt((dx)**2 + (dy)**2)

    maskdist = (dist <= distmax ) 

    galcomps.Active[maskdist] = True

    return galcomps
 



def conver2Sersic(galcomps: GalComps) -> GalComps:
    ''' function to convert exponential, gaussian params to Sersic params'''

    comps =  copy.deepcopy(galcomps)

    maskdev = comps.NameComp == "devauc"
    maskexp = comps.NameComp == "expdisk"
    maskgas = comps.NameComp == "gaussian"


    K_GAUSS = 0.6931471805599455 #constant k for gaussian
    K_EXP = 1.6783469900166612 # constant k for expdisk
    SQ2 = np.sqrt(2) 

    #for gaussian functions
    if maskgas.any():
        comps.Exp[maskgas] = 0.5 
        comps.Rad[maskgas] = comps.Rad[maskgas]/2.354 #converting to sigma 
        comps.Rad[maskgas] = SQ2*(K_GAUSS**0.5)*comps.Rad[maskgas] #converting to Re 


    #for de vaucouleurs
    if maskdev.any():
        comps.Exp[maskdev] = 4

    #for exponential disks
    if maskexp.any():
        comps.Exp[maskexp] = 1
        comps.Rad[maskexp] = K_EXP*comps.Rad[maskexp] #converting to Re


    return comps



class GetSlope:
    '''class to obtain the effective radius for the whole galaxy'''



    def FullSlopeSer(self, R: float, Re: list, n: list, q: list, pa: list, theta: float) -> float:


        SlptotR = self.SlopeSer(R, Re, n, q, pa, theta) 

        return SlptotR.sum()


        maskgal = (comps.Active == True) #using active components only 

        gam = np.array([])

        for r in R:

            slp = self.SlopeSer(r, comps.Ie[maskgal], comps.Rad[maskgal], comps.Exp[maskgal], comps.AxRat[maskgal], comps.PosAng[maskgal], theta)
            gam = np.append(gam, slp)



        return gam


    def funGalSlopeSer(self, R: float, Ie: list, Re: list, n: list, q: list, pa: list, theta: float, slope: float) -> float:


        fun = self.SlopeSer(R, Ie, Re, n, q, pa, theta)  - slope


        return fun
     



    def FindSlope(self, comps: GalComps, theta: float, slope: float) -> float:
        "return the Re of a set of Sersic functions. It uses Bisection"

        maskgal = (comps.Active == True) #using active components only 

        a = 0.1 
        b = comps.Rad[maskgal][-1] * 10  # hope it doesn't crash

        try:
            Radslp = bisect(self.funGalSlopeSer, a, b, args=(comps.Ie[maskgal], comps.Rad[maskgal], comps.Exp[maskgal], comps.AxRat[maskgal], comps.PosAng[maskgal], theta, slope))
        except:
            print("solution not found in given range")
            Radslp=0

        return Radslp 


    def var_X(self, R: float, Re: list, n: list):

        k = gammaincinv(2*n, 0.5)

        varx = k*((R/Re)**(1/n) - 1)


        return varx

    def var_Xprim(self, R: float, Re: list, n: list):

        k = gammaincinv(2*n, 0.5)

        varxpr = (k/n)*((R/Re)**(1/n - 1))*(1/Re) 


        return varxpr

    def var_S(self, R: float, Ie: list,  Re: list, n: list, X: list):

        S = Ie*np.exp(-X)

        return S.sum()

    def var_Sprim(self, R: float, Ie: list,  Re: list, n: list, X: list, Xprim: list):

        Sprim = Ie*np.exp(-X)*Xprim

        return Sprim.sum()

    def SlopeSer(self, R: float, Ie: list, Re: list, n: list, q: list, pa: list, theta: float) -> float:
        '''slope from sersic function to a determined R'''
            
        Rcor = GetRadAng(R, q, pa, theta) 

        X = self.var_X(Rcor, Re, n)
        Xprim = self.var_Xprim(Rcor, Re, n)

        S = self.var_S(Rcor, Ie,  Re, n, X)
        Sprim = self.var_Sprim(Rcor, Ie,  Re, n, X, Xprim)

        Slp = (Sprim/S)*R 


        return Slp




def GetRadAng(R: float, q: list, pa: list, theta: float) -> float:
    '''Given an ellipse and an angle it returns the radius in angle direction. 
    Theta are the values for the galaxy and the others for every component'''


    #changing measured angle from y-axis to x-axis
    # and changing to rads:
    newpa = (pa + 90)*np.pi/180 #angle of every component
    theta = (theta + 90)*np.pi/180 #angle of direction of R 

    #bim = q * R

    ecc = np.sqrt(1 - q**2)

    alpha = theta - newpa #this is the direction 


    bell =  R*np.sqrt(1 - (ecc*np.cos(alpha))**2)


    aell = bell/q  #rad to evalue for every component



    return aell 







#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  main()



